using System;
using Toub.Sound.Midi;

namespace WindowsApplication5
{
	/// <summary>
	/// Summary description for MidiBand.
	/// </summary>
	public class MidiBand
	{
		// Public Variables
		public static byte PitchNextNote = 32;
		public static byte PitchMax = 255;
		public static byte PitchBase = 127;

		// Private Variables
		public byte MIDIChannel;
		private byte MIDIDelay;
		public Toub.Sound.Midi.GeneralMidiInstruments MIDIInstrument;
		private byte Pitch;
		
		private byte midinote;
		public byte MIDINote 
		{
			get 
			{
				return midinote;
			}
			set 
			{
				if (value <= 127)
					midinote = value;
				else 
					midinote = 127;
			}
		}
		public byte MIDINoteBase;
		private byte MIDIOldVelocity;

		private int LengthMin;
		public int LengthMax;
		private int Length;
		
		// Constructors and Destructors
		public MidiBand(byte midichannel, byte mididelay, Toub.Sound.Midi.GeneralMidiInstruments midiinstrument,
			byte midinotebase, int lengthmin, int lengthmax)
		{
			MIDIChannel = midichannel;
			MIDIDelay = mididelay;
			MIDIInstrument = midiinstrument;
			Pitch = PitchBase;
			MIDINote = MIDINoteBase = midinotebase;
			LengthMin = lengthmin;
			LengthMax = lengthmax;
			MIDIOldVelocity = 0;
			MidiPlayer.OpenMidi();
			MidiPlayer.Play(new PitchWheel(MIDIDelay, MIDIChannel, PitchBase));
			SetInstrument(MIDIInstrument);
		}

		public void SetInstrument(Toub.Sound.Midi.GeneralMidiInstruments ins) 
		{
			MidiPlayer.Play(new ProgramChange(MIDIDelay, MIDIChannel,ins) ); 
			MIDIInstrument = ins;
		}

		~MidiBand() 
		{
			MidiPlayer.CloseMidi();
		}

		public void PluckRaw(byte velocity) 
		{
			if (MIDIOldVelocity != 0)
				MidiPlayer.Play(new NoteOff(MIDIDelay,MIDIChannel, MIDINote, MIDIOldVelocity));

			MidiPlayer.Play(new PitchWheel(MIDIDelay, MIDIChannel, Pitch));
			MidiPlayer.Play(new NoteOn(MIDIDelay, MIDIChannel, MIDINote, velocity) );
			MIDIOldVelocity = velocity;
		}

		public byte CalcNote() 
		{
			return ((byte) ((Length-LengthMin) / PitchNextNote + MIDINoteBase));
		
		}

		public void Pluck(byte velocity) 
		{
			if (MIDIOldVelocity != 0)
				MidiPlayer.Play(new NoteOff(MIDIDelay,MIDIChannel, MIDINote, MIDIOldVelocity));

			MIDINote = CalcNote();
			Pitch = (byte) ( ((Length-LengthMin) % PitchNextNote) + PitchBase );
			MidiPlayer.Play(new PitchWheel(MIDIDelay, MIDIChannel, Pitch));
			MidiPlayer.Play(new NoteOn(MIDIDelay, MIDIChannel, MIDINote, velocity) );
			MIDIOldVelocity = velocity;
		}

		
		public bool Stretch(int length) 
		{
			if (length < LengthMin || length > LengthMax)
				return false;
			Length = length;
			length -= LengthMin;
			length -= (MIDINote - MIDINoteBase) * PitchNextNote;
			length += PitchBase;
			if (length > PitchMax) length = PitchMax;
			if (length < 0) length = 0;
			MidiPlayer.Play(new PitchWheel(MIDIDelay, MIDIChannel, length));
			return true;
		}
	}
}
