using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace WindowsApplication5
{
	public class RubberBand : Sdgt.SdgUserControl
	{

		// Properties
		private byte Channel;
		[ Category ( "User Defined" ) , Description ("Selects the MIDI Channel") ]
		public byte MIDIChannel
		{
			get
			{
				return Channel;
			}
			set
			{
				Channel = value;
				if (!InDesign) BandNoise.MIDIChannel = value;
			}
		}


		private int lengthmin;
		[ Category ( "User Defined" ) , Description ("Minimum Length of the band") ]
		public int LengthMin
		{
			get
			{
				return lengthmin;
			}
			set
			{
				lengthmin = value;
			}
		}


		private byte NoteBase;
		[ Category ( "User Defined" ) , Description ("Selects the lowest MIDI note for this band") ]
		public byte MIDINoteBase
		{
			get
			{
				return NoteBase;
			}
			set
			{
				NoteBase = value;
				if (!InDesign) BandNoise.MIDINoteBase = value;
			}
		}


		private System.Drawing.Color color;
		[ Category ( "User Defined" ) , Description ("Selects the band Color") ]
		public System.Drawing.Color Color
		{
			get
			{
				return color;
			}
			set
			{
				color = value;
			}
		}


		private Toub.Sound.Midi.GeneralMidiInstruments Instrument;
		[ Category ( "User Defined" ) , Description ("Selects the MIDI Instrument") ]
		public Toub.Sound.Midi.GeneralMidiInstruments MIDIInstrument 
		{
			get 
			{
				return Instrument;
			}
			set 
			{
				Instrument = value;
				if (!InDesign) BandNoise.SetInstrument(value);
			}
		}
			
		public bool InDesign 
		{
			get 
			{
			  return (System.Diagnostics.Process.GetCurrentProcess().ProcessName == "devenv");
			}
		}


		// For dragging the pins
		private static int PINS = 2;
		private int [] PinDraggedBy;
		private int [] PinXOff;
		private Image [] PinsUp;
		private Image [] PinsDown;

		// For the Line/Arc
		private Graphics canvas;
		private int ArcHeight;
		private int HarmonicMouse;
		private int HarmonicFuzzyRange = 7;
		private int ArcDir;
		private int Speed;  // How fast the band moves. Relative to the length
		private float Volume; // The height of the band. Slowly decays. Percentage (0..1)
		private float Decay = 0.05f;
		private static int SpeedMax = 50;
		private static int SpeedMin = 20;

		private MidiBand BandNoise;

		private int Length;
		private int LengthMax;
		private static int WidthMax = 10;
		private static int WidthMin = 1;
		private static int MICE = 4;

		// For detecting which side of the band we are on.
		private enum Side 
		{
			Unknown,
			Top,
			Bottom
		}
		private Side[] MouseSide;
	
		private System.Windows.Forms.PictureBox[] Pins;
		private System.Windows.Forms.PictureBox pin2;
		private System.Windows.Forms.PictureBox pin1;
		private System.Windows.Forms.Timer timer1;
		private System.Windows.Forms.PictureBox pin1up;
		private System.Windows.Forms.PictureBox pin2up;
		private System.ComponentModel.IContainer components = null;

		public RubberBand()
		{
			// This call is required by the Windows Form Designer.
			InitializeComponent();
			
			
			PinDraggedBy = new int[PINS];
			PinXOff = new int[PINS];
			MouseSide = new Side[MICE];
			PinsUp = new Image[PINS];
			PinsDown = new Image[PINS];

			Pins = new System.Windows.Forms.PictureBox[PINS];
			for (int i = 0; i < PINS; i++) 
				PinDraggedBy[i] = -1;
			for (int i = 0; i < MICE; i++) 
				MouseSide[i] = Side.Unknown;
			

			Pins[0] = pin1;
			Pins[1] = pin2;
			PinsUp[0] = (Image)pin1up.Image.Clone();
			PinsUp[1] = (Image)pin2up.Image.Clone();
			PinsDown[0] = (Image)pin1.Image.Clone();
			PinsDown[1] = (Image)pin2.Image.Clone();
			
			ArcHeight = 0;
			ArcDir = 1;
			canvas = this.CreateGraphics();

			LengthMax = Pins[1].Left - Pins[0].Right;
			if (!InDesign)
			  BandNoise = new MidiBand(Channel, 0, Instrument, NoteBase, LengthMin, LengthMax );
			HarmonicMouse = -1;
			SetupLength();
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(RubberBand));
			this.pin2 = new System.Windows.Forms.PictureBox();
			this.pin1 = new System.Windows.Forms.PictureBox();
			this.timer1 = new System.Windows.Forms.Timer(this.components);
			this.pin1up = new System.Windows.Forms.PictureBox();
			this.pin2up = new System.Windows.Forms.PictureBox();
			this.SuspendLayout();
			// 
			// pin2
			// 
			this.pin2.Image = ((System.Drawing.Image)(resources.GetObject("pin2.Image")));
			this.pin2.Location = new System.Drawing.Point(160, 0);
			this.pin2.Name = "pin2";
			this.pin2.Size = new System.Drawing.Size(32, 50);
			this.pin2.SizeMode = System.Windows.Forms.PictureBoxSizeMode.StretchImage;
			this.pin2.TabIndex = 1;
			this.pin2.TabStop = false;
			// 
			// pin1
			// 
			this.pin1.Image = ((System.Drawing.Image)(resources.GetObject("pin1.Image")));
			this.pin1.Location = new System.Drawing.Point(0, 0);
			this.pin1.Name = "pin1";
			this.pin1.Size = new System.Drawing.Size(32, 50);
			this.pin1.SizeMode = System.Windows.Forms.PictureBoxSizeMode.StretchImage;
			this.pin1.TabIndex = 2;
			this.pin1.TabStop = false;
			// 
			// timer1
			// 
			this.timer1.Interval = 30;
			this.timer1.Tick += new System.EventHandler(this.timer1_Tick);
			// 
			// pin1up
			// 
			this.pin1up.Image = ((System.Drawing.Image)(resources.GetObject("pin1up.Image")));
			this.pin1up.Location = new System.Drawing.Point(0, 0);
			this.pin1up.Name = "pin1up";
			this.pin1up.Size = new System.Drawing.Size(32, 50);
			this.pin1up.SizeMode = System.Windows.Forms.PictureBoxSizeMode.StretchImage;
			this.pin1up.TabIndex = 3;
			this.pin1up.TabStop = false;
			this.pin1up.Visible = false;
			// 
			// pin2up
			// 
			this.pin2up.Image = ((System.Drawing.Image)(resources.GetObject("pin2up.Image")));
			this.pin2up.Location = new System.Drawing.Point(160, 0);
			this.pin2up.Name = "pin2up";
			this.pin2up.Size = new System.Drawing.Size(32, 50);
			this.pin2up.SizeMode = System.Windows.Forms.PictureBoxSizeMode.StretchImage;
			this.pin2up.TabIndex = 4;
			this.pin2up.TabStop = false;
			this.pin2up.Visible = false;
			// 
			// RubberBand
			// 
			this.BackColor = System.Drawing.SystemColors.ActiveBorder;
			this.Controls.Add(this.pin2up);
			this.Controls.Add(this.pin1up);
			this.Controls.Add(this.pin1);
			this.Controls.Add(this.pin2);
			this.Name = "RubberBand";
			this.Size = new System.Drawing.Size(192, 100);
			this.Resize += new System.EventHandler(this.RubberBand_Resize);
			this.Leave += new System.EventHandler(this.RubberBand_Leave);
			this.SdgMouseMove += new Sdgt.SdgMouseEventHandler(this.RubberBand_SdgMouseMove);
			this.SizeChanged += new System.EventHandler(this.RubberBand_SizeChanged);
			this.SdgMouseDown += new Sdgt.SdgMouseEventHandler(this.RubberBand_SdgMouseDown);
			this.SdgMouseUp += new Sdgt.SdgMouseEventHandler(this.RubberBand_SdgMouseUp);
			this.Paint += new System.Windows.Forms.PaintEventHandler(this.RubberBand_Paint);
			this.ResumeLayout(false);

		}
		#endregion

		private bool hitbox(int x, int y, int boxX, int boxY, int boxWidth, int boxHeight) 
		{
			return( x >= boxX && x <= (boxX+boxWidth) &&
				 y >= boxY && y <= (boxY+boxHeight));
		}

		private void RubberBand_SdgMouseDown(object sender, Sdgt.SdgMouseEventArgs e)
		{
			// Test if we hit one of the pins
			for (int i=0; i < PINS; i++)
				if (hitbox(e.X, e.Y, Pins[i].Left, Pins[i].Top, Pins[i].Width, Pins[i].Height)) 
				{
					if (PinDraggedBy[i] == -1)
					{
						PinDraggedBy[i] = e.ID;
						PinXOff[i] = Pins[i].Left - e.X;
						Pins[i].Image = PinsUp[i];
					}
				}
		
		}

		private Side GetSide(int x, int y) 
		{
			if (x < Pins[0].Right || x > Pins[1].Left)
				return Side.Unknown;

			if (y < Pins[0].Bottom) 
				return Side.Top;
			else
				return Side.Bottom;
		}

		private void RubberBand_SdgMouseMove(object sender, Sdgt.SdgMouseEventArgs e)
		{
			// Pin Movement
			for (int i=0; i<PINS; i++)
				if (PinDraggedBy[i] == e.ID) 
				{
					int newlen;
					if (i == 0)
						newlen = Pins[1].Left - (e.X + PinXOff[i] + Pins[0].Width);
					else
						newlen = (e.X+PinXOff[i]) - Pins[0].Right;
					if (newlen > LengthMax || newlen < LengthMin)
						return;
					Length = newlen;
					Speed = (int)(((float)newlen / (float)LengthMax)*(SpeedMax-SpeedMin)+SpeedMin);
					BandNoise.Stretch(newlen);
					Pins[i].Left = e.X + PinXOff[i];
				}
			
			// == Mouse strumming

			
			Side side = GetSide(e.X, e.Y);
			
			// Test for Harmonics
			if (HarmonicArea(e.X, e.Y, e.ID, e.Button))
				HarmonicMouse = e.ID;
			else
				if (HarmonicMouse == e.ID)
				HarmonicMouse = -1;

			if (MouseSide[e.ID] == Side.Unknown || side == Side.Unknown) 
				MouseSide[e.ID] = side;
			else if (MouseSide[e.ID] != side && side != Side.Unknown) 
			{
				//Test for 1st harmonic
				if (HarmonicMouse != -1)  // We have harmonic position
				{
					if (HarmonicMouse != e.ID) // The OTHER mouse is holding the harmonic posn
					{	
						// Manually set the midinote
						BandNoise.MIDINote = (byte)(BandNoise.CalcNote()*2);
						BandNoise.PluckRaw((byte)(127));
						Volume = 1;
						timer1.Enabled = true;
					}	
				} // No harmonic position
				else
				{
					BandNoise.Pluck(127);
					Volume = 1; 
					timer1.Enabled = true;
				}
				MouseSide[e.ID] = side;
			}
		}

		private bool HarmonicArea(int x, int y, int id, System.Windows.Forms.MouseButtons btn) 
		{
			int halfX = (Pins[1].Left + Pins[0].Right) /2;
			int center = Pins[1].Bottom;
			return ( ( (btn & MouseButtons.Left) > 0 || HarmonicMouse==id) &&
					x < halfX+HarmonicFuzzyRange && x > halfX-HarmonicFuzzyRange &&
					y < center+HarmonicFuzzyRange && y > center-HarmonicFuzzyRange);
		}

		private void RubberBand_SdgMouseUp(object sender, Sdgt.SdgMouseEventArgs e)
		{
			for (int i=0; i< PINS; i++)
				if (PinDraggedBy[i] == e.ID) 
				{
					PinDraggedBy[i] = -1;
					Pins[i].Image = PinsDown[i];
				}
		
		}

		private void DrawBand() 
		{
			
			if (this.BackgroundImage == null)
				canvas.Clear(this.BackColor);
			else 
				canvas.DrawImage(this.BackgroundImage,0,0,this.Width,this.Height);
			
			if (Length == 0)
				return;

			int wid = (int) (((float)(LengthMax - Length) / (float)LengthMax) * (WidthMax-WidthMin)+WidthMin);
			Pen MyPen = new Pen(color, wid);

			if (ArcHeight == 0) 
			{
				canvas.DrawLine(MyPen, new Point(Pins[0].Right, Pins[0].Bottom), 
					new Point(Pins[1].Left, Pins[1].Bottom));
				return;
			}
			if (HarmonicMouse == -1)
				canvas.DrawArc(MyPen, Pins[0].Right, Pins[0].Bottom-ArcHeight, (Pins[1].Left - Pins[0].Right),
					ArcHeight*2, 180*ArcDir, -1*180*ArcDir);
			else 
			{
				int midX = (Pins[1].Left+Pins[0].Right)/2;
				int leftX = Pins[0].Right;
				int rightX = Pins[1].Left;
				int widthX = midX-leftX;

				canvas.DrawArc(MyPen, leftX, Pins[0].Bottom-ArcHeight/2, widthX, ArcHeight, 180*ArcDir, -1*180*ArcDir);
				canvas.DrawArc(MyPen, midX, Pins[0].Bottom-ArcHeight/2, widthX, ArcHeight, -1*180*ArcDir, 180*ArcDir);
			}
		}

		int dir = 1;
		private void timer1_Tick(object sender, System.EventArgs e)
		{
			if (Volume > 0) 
			{
				ArcHeight += dir * Speed;
				int height = (int)(Volume*Pins[0].Height);
				if (ArcHeight > height) ArcHeight = height;
				if (ArcHeight < 0) ArcHeight = 0;
				if (ArcHeight == height || ArcHeight == 0)
					dir *= -1;

				if (ArcHeight == 0) 
					ArcDir *= -1;
				Volume -= Decay;
			} 
			else
			{
				dir = 1; // Restart direction of pluck.
				timer1.Enabled = false;
				ArcHeight = 0;
				Volume = 0;
			}
		DrawBand();
		}

		private void RubberBand_Resize(object sender, System.EventArgs e)
		{
			Pins[1].Left = this.Width-Pins[1].Width;
			Length = LengthMax = Pins[1].Left - Pins[0].Right;
			canvas = this.CreateGraphics();
			DrawBand();
		}

		private void SetupLength() 
		{
			Length = LengthMax = Pins[1].Left - Pins[0].Right;
			if (!InDesign) 
			{
				BandNoise.LengthMax = LengthMax;
				BandNoise.Stretch(LengthMax);
				Speed = SpeedMax;
			}
		}

		private void RubberBand_SizeChanged(object sender, System.EventArgs e)
		{
			SetupLength();
		}

		private void RubberBand_Leave(object sender, System.EventArgs e)
		{
			for (int i = 0; i < PINS; i++)
				MouseSide[i] = Side.Unknown;
		}

		private void RubberBand_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			DrawBand();
		}
	}
}

